/* This is the exported header file for the dynamic link library FE813USB.DLL	*/
/*																				*/
/* Author : Martin Stringfellow													*/
/* Copyright : Fylde Electronic Laboratories Ltd.								*/
/* This file is intended for distribution to software developers wishing to		*/
/* use the DLL functions. No warranty is expressed by the comments herein.		*/
/*																				*/
/*																				*/
/* The DLL is built for 32 bit Windows operating systems such as XP and W2k		*/
/* Standard calling convention is used for exported functions.						*/
/*																				*/
/* Unles otherwise stated successful functions return TRUE.						*/
/*																				*/
/* Modification History															*/
/*																				*/
/* Issue	Date	Comment														*/
/*	1		21/3/03	Tested for use with DASYLab Driver.							*/
/*	2		22/4/03 Get_Blocks_Transferred now returns a long integer (DWORD)	*/
/*  3		8/5/03  Aditional functions for use by LabView.						*/
/*  4       6/11/03 Additional functions :										*/
/*				DWORD Get_Sample_Modulus(int Channel)							*/
/*				void Set_Sample_Boundary(int Modulus_Multiples)					*/
/*				int Get_FE813USB_Status(void)									*/
/*				DWORD Get_Current_DataPoint(int Channel)						*/
/*	5		16/1/08 Additional Function :										*/
/*				char* Get_String(void) --	Gets an error string or empty		*/
/*											string if no error					*/
/*  6		24/6/08  Pause_Acquisition() and RestartAcquisition() added
/*																				*/
/* Function Descriptions														*/
//////////////////////////////////////////////////////////////////////////////////									
/////////////////////////////////////////////////////////////////////////////////
// int Get_Sample_Rate (int Channel)											
// ////////////////////////////////-											
// This function just returns the value (50,25,12,10,5,2,1,0) which is indicates
// the sample rate for the channel. The Channel parameter is in the range 0..7	
//																				
/////////////////////////////////////////////////////////////////////////////////
// BOOL Start_Acquisition(HWND hwndMain)										
// ////////////////////////////////////-										
// Ths function finds starts a thread which finds the USB device and initialises
// it with the sample rates. It then reads data continuously from the USB device
// into private buffers maintained by the DLL. There are 64 buffers each holding	
// 16384 samples. They are written and then overwritten in sequence. Hence the 	
// data must be used by a client process before it is overwritten. The parameter
// is the handle of the window of the calling process. It is needed for Windows	
// to know which process owns the thread. The function returns TRUE if the data	
// acquiistion thread started successfuly.										
// The LV_Start_Acquisistion version was introduced to support the LabView      
// driver which does not support a window handle as a parameter. A handle is    
// obtained for the current foreground window and used to start the thread.     
//																				
/////////////////////////////////////////////////////////////////////////////////
// void Stop_Acquisition()														
// ///////////////////////
// This functions stops the data acquisistion thread. The thread will end. 
// A paused thread can be stopped without restarting
// 						
/////////////////////////////////////////////////////////////////////////////////
// void Pause_Acquisition()														
// ///////////////////////
// This functions pauses the data acquisistion thread. The thread does not end but
// waits for a signal to restart. A paused thread can be stopped or restarted.
// 						
//////////////////////////////////////////////////////////////////////////////////
// BOOL Restart_Acquisition()														
// ///////////////////////
// This functions restarts the paused data acquisistion thread and returns TRUE. 
// If the thread is not paused, the function has no effect except it returns FALSE.  
// 						
//////////////////////////////////////////////////////////////////////////////////
// int Get_Blocks_Transferred()													
// ////////////////////////////													
// This function returns a value which represents the number of bocks of 32000	
// samples transferred from the USB device to the DLL's internal buffers. 		
//																				
//////////////////////////////////////////////////////////////////////////////////
// int Get_DataPoint(int Channel , DWORD DataPoint)								
// ////////////////////////////////////////////////								
// This function returns a (16 bit) data sample transferred from the USB device.
// The DataPoint parameter is the index of the data point starting at zero.		
// The caller must have a concept of time passed since the start of acquisition 
// and a knowledge of the sample rate for the channel in order to be sure that  
// the sample is available.	(The DLL function Get_Current_DataPoint(Channel) may be 
// used to determine the index to the most recent data point.
// The channel parameter must be in the range 0..4. The data point is a positive
// integer in the range 0..65535. 0 represents -10 V and 65535 represents +10 V.
// A negative result is an error as follows :									
//																				
// FE813USB_ERROR_NO_TRANSFERS : No DLL internal buffers have yet been filled.	
// You would get this error if you tried to read a data point immediately after	
// starting the data acquisistion thread.										
//																				
// FE813USB_ERROR_NO_CHANNELS : The channel specified has a sample rate of zero.
// i.e. it is OFF.																
//																				
// FE813USB_ERROR_SAMPLE_NOT_READY : The data point specified has not yet been 	
// transferred from the USB device to the DLL's internal buffers.				
//																				
// FE813USB_ERROR_SAMPLE_DISCARDED : The data point specified has been			
// overwritten in the DLL's internal buffers.									
//                                                                              
////////////////////////////////////////////////////////////////////////////////////////
// BOOL Do_Autozero(void)														
// //////////////////////														
// This function will initiate the auto zero of all FE-366-TA modules (if any   
// are fitted. It returns immediately although the Auto Zero
// typically takes 5 seconds to complete.
//	
////////////////////////////////////////////////////////////////////////////////////////
// BOOL Do_Cal_On(void)															
// ////////////////////                                                         
// This funtion sets the shunt calibration output ON. This signals all			
// FE-366-TA modules to apply shunt calibration (if they are configured with    
// calibration resistors.														
//																				
////////////////////////////////////////////////////////////////////////////////////////
// BOOL Do_Cal_Off(void)														
// ////////////////////-														
// This function turns OFF the calibration output.								
//									
////////////////////////////////////////////////////////////////////////////////////////
// BOOL LV_Start_Acquisition(void)												
// //////////////////////////////											
// This function is the same as "Start_Acquisistion" above, execept that there  
// is no window handle parameter. The function is intended for use by languages 
// such as LabView which do not know how to pass a window handle as a parameter 
// to a DLL. The function needs a window handle to create the data acquisition  
// thread, so it uses the handle to the current foreground window.				
//																																						
////////////////////////////////////////////////////////////////////////////////////////
// int Get_Data_Samples(int *SampleArray, int Channel , DWORD FirstPoint,		
//			int NumberOfSamples)
// //////////////////////////////////////////////////////////////////////////   																			
// Parameters :																	
// *SampleArray -> Pointer to an array of 16 bit signed integers				
// int Channel -> Represents the channel for reading. (in the range 0..7)		
// DWORD FirstPoint -> Index to the first data point to read. Zero if this is   					
//						the first sample read after acquisition started.        
//                      (32 bit unsigned integer)								
// int NumberOfSamples -> The length of the array (or the number of samples to be    
//                    written to the array. (32 bit integer)                     
//																				
// This function returns the number of data points tranferred to the array.     
//																				
////////////////////////////////////////////////////////////////////////////////////////
//int Get_Trigger(int *SampleArray,int Channel,DWORD FirstPoint,int NumberOfSamples,
//									  int EdgeType,WORD TriggerLevel
//////////////////////////////////////////////////////////////////////////////	
// Parameters :																	
// *SampleArray -> Pointer to an array of 16 bit unsigned integers				
// int Channel -> Represents the channel for reading. (in the range 0..7)		
// DWORD FirstPoint -> Index to the first data point to read. Zero if this is   					
//						the first sample read after acquisition started.        
//                      (32 bit unsigned integer)								
// int NumberOfSamples -> The length of the array (or the numbr of samples to be    
//                    written to the array. (32 bit integer)
// int EdgeType->	0 : RisingEdge
//					1 : FallingEdge
//					2 : Above Level
//					3 : Below Level
// WORD TriggerLevel-> The level at which the trigger is set. (Unsigned 16 bit integer)
// Return Value is zero if no trigger found and is the index to the trigger point in
// the array otherwise.
//
//
////////////////////////////////////////////////////////////////////////////////////////
// DWORD Get_Sample_Modulus(int Channel)
/////////////////////////////////////////
// The USB system transfers all samples from all channels in fixed size blocks.
// These blocks are then moved into a large buffer.
// The samples for each channel are mixed up in the buffer which means that to extract a sample
// from the buffer the DLL needs to know how many samples have been taken since the
// start of data acquisition. 
// However after a large number of samples have been taken, the pattern repeats, so the 
// DLL only needs to know how many samples have been taken since the pattern began to repeat.
//
// The Sample Modulus for a channel is the number of samples which occur before the pattern
// of samples in the buffer repeats.
//
// The DLL user can therefore keep count of the number of samples taken from a channel 
// by counting up to the sample modulus, and each time the count exceeds the sample 
// modulus resetting the count.
//
////////////////////////////////////////////////////////////////////////////////////////
//
// void Set_Sample_Boundary(int Modulus_Multiples)
//////////////////////////////////////////////////
// Each time the count of a number of samples taken from a channel exceeds the sample
// modulus, the user can call this function and then subtract the sample modulus from
// the total count. This helps to prevent the sample count from overflowing the 32 bit
// parameter which is used to get each data point.
//
// This function does not have to be used. It is needed only in applications where the
// total number of samples read from a channel might exceed the maximum value of a
// 32 bit integer.
//
////////////////////////////////////////////////////////////////////////////////////////
// int Get_FE813USB_Status(void)
////////////////////////////////
// This function is useful when you need to check if the device is present or if
// a download of code to the DSP is required. When called after a download of code,
// it returns a positive integer which is the number of blocks downloaded
//
// Otherwise it returns 
// FE813USB_ERROR_NO_DEVICE (when no USB device is present)
// FE813USB_ERROR_NO_DC_POWER (when the Device is present but not powered)
//
///////////////////////////////////////////////////////////////////////////////////////
//DWORD Get_Current_DataPoint(int Channel)
//////////////////////////////////////////
// This function returns the index to the most recently acquired data point for a channel.
// It does not return the value of the data point. 
////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////////////
// *char Get_String(void) 
/////////////////////////
// This function is only for the USB 2.0 CyUSB.sys driver.
// It returns an empty string if there was no error with the data transfer.
// Otherwise it returns an error description.
////////////////////////////////////////////////////////////////////////////////////////
// BOOL Set_Calibration_Data(DWORD Cal_Data)
///////////////////////////////////////////
// This function writes calibration data to an EEPROM.
// Returns TRUE if success
// Calibration data is used to convert the 16 bit data value to Volts
// nanoVolts = (Reading - 32768) x (Scale) x 10-9 where Scale is approximately 312500
// e.g. Full Scale 0xFFFF is 10.24V and 0x0000 is -10.24 V
// All channels use the same scaling 
// The stored calibration data is 312500 +/- 0.2% 
// It corrects for the accuracy of the A to D reference which is 4.096V +/- 10 mV
////////////////////////////////////////////////////////////////////////////////////////
// DWORD Get_Calibration_Data(void)
///////////////////////////////////
// Returns a value of 312500 +/- 0.2% (See above)
////////////////////////////////////////////////////////////////////////////////////////
#ifndef __FE813USBLIB_H
#define __FE813USBLIB_H
#ifndef __FE813USB__
#define __FE813USBLIB__ __declspec(dllimport)
#else
#define __FE813USBLIB__ __declspec(dllexport)
#endif
#ifdef __cplusplus
extern "C" {
#endif
#define WINAPI __stdcall
__FE813USBLIB__ BOOL  WINAPI Start_Acquisition(HWND hwndMain);
__FE813USBLIB__ void  WINAPI  Stop_Acquisition();
__FE813USBLIB__ BOOL  WINAPI  Restart_Acquisition();
__FE813USBLIB__ void  WINAPI  Pause_Acquisition();
__FE813USBLIB__ DWORD  WINAPI  Get_Blocks_Transferred();
__FE813USBLIB__ int  WINAPI  Get_DataPoint(int Channel , DWORD DataPoint);
__FE813USBLIB__ void  WINAPI  Set_Sample_Rate (int Channel, int Sample_Rate);
__FE813USBLIB__ int  WINAPI  Get_Sample_Rate (int Channel);
__FE813USBLIB__ BOOL  WINAPI  Do_Autozero(void);
__FE813USBLIB__ BOOL  WINAPI  Do_Cal_On(void);
__FE813USBLIB__ BOOL  WINAPI  Do_Cal_Off(void);
__FE813USBLIB__ BOOL  WINAPI  LV_Start_Acquisition(void);

__FE813USBLIB__ int  WINAPI  Get_Data_Samples(int *SampleArray , //pointer to array
									 int Channel , 
									 DWORD FirstPoint, 
									 int NumberOfSamples);
__FE813USBLIB__ int  WINAPI  Get_Trigger(int *SampleArray, // pointer to array
								int Channel,
								DWORD FirstPoint,
								int NumberOfSamples,
								int EdgeType,
								WORD TriggerLevel);
__FE813USBLIB__ DWORD  WINAPI  Get_Sample_Modulus(int Channel);
__FE813USBLIB__ void  WINAPI  Set_Sample_Boundary(int Modulus_Multiples);
__FE813USBLIB__ int  WINAPI  Get_FE813USB_Status(void);
__FE813USBLIB__ DWORD  WINAPI  Get_Current_DataPoint(int Channel);
__FE813USBLIB__ BOOL  WINAPI  Set_Calibration_Data(DWORD Cal_Data);
__FE813USBLIB__ DWORD  WINAPI  Get_Calibration_Data(void);




#ifdef __cplusplus
}
#endif
#define FE813USB_ERROR_NO_TRANSFERS		-1
#define FE813USB_ERROR_NO_CHANNELS		-2
#define FE813USB_ERROR_SAMPLE_NOT_READY	-3
#define FE813USB_ERROR_SAMPLE_DISCARDED	-4
#define FE813USB_ERROR_NO_DEVICE		-5
#define FE813USB_ERROR_NO_DSP_CODE		-6
#define FE813USB_ERROR_NOT_VALID_DEVICE -7
#define FE813USB_ERROR_NO_DC_POWER		-8
#endif
