// FE813USB_Test.cpp : Defines the entry point for the console application.
//
// Modified to include Pause and Restart of Data Acquisition : 24/6/08
// Modified for FE-813-USB

#include "stdafx.h"				// Automatically included for VC++ console applications
#include "FE813USB_Test.h"		// Automatically included for VC++ console applications
#include "windows.h"			// Required for waitable timer
#include "conio.h"				// Required so that keyboard input can be read.
#include "FE813USBLIB.h"		// Links to DLL

// The one and only application object

CWinApp theApp;

using namespace std;

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	int Modulus_Multiples = 0;	// Ticks up, every time we reset the sample index to 
	int i = 0;					// Used for loop indexing 
	int k = 0;
	int iSrate[4] = {5,5,5,5};	// Sample Rate enumerator 50 kHz 
	int A_channel = 0;					// Channel number 0..7

	BOOLEAN CAL_state = FALSE;		// The state of the Shunt Cal output

	DWORD li_Analog_Index[4] = {0,0,0,0}; // Counts the number of samples taken from each channel
	int iData_From_Channel[4] = {32768,32768,32768,32768}; // The data read from each channel
	BOOLEAN New_Data;	// set TRUE when new data is available from any channel		
	HANDLE hTimer = NULL;	// Used to create and access waitable timer
	LARGE_INTEGER liDueTime;	// Used by waitable timer
	liDueTime.QuadPart=-250000;	// 25 ms wait for waitable timer
	DWORD Cal_Data;
	float Cal_Multiplier;


	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		// change error code to suit your needs
		_tprintf(_T("Fatal Error: MFC initialization failed\n"));
		nRetCode = 1;
	}
	else
	{
		printf ("Fylde Electronic Laboratories\n");
		printf ("Test Program for FE-813-USB\n");
		printf ("Version 1.0 8/6/2010\n");
		printf ("This program displays every 10,000th sample from each channel\n\n");

		if (Get_FE813USB_Status() == 101) // 101 is number of blocks downloaded (if successful)
		{
			printf ("Device Ready.\n"); 

				// Set initial digital output states
			Do_Cal_Off();

			// Remind User that this writes the EEPROM with default Calibration data


			Cal_Data = Get_Calibration_Data();
			if ( (Cal_Data < 320000) && (Cal_Data > 300000) ) {
				printf ("\n\rCal Data set to : %d\n\r",Cal_Data); 

			}
			else {
				printf ("\n\rCalibration Data could not be read from EEPROM\n");
				// Use default
				Cal_Data = 312500;
			}
			Cal_Multiplier = float(Cal_Data) / 1000000000;

			// Create a waitable timer which we will use when data is not ready
			hTimer = CreateWaitableTimer(NULL, TRUE, "WaitableTimer");
			if (!hTimer)
			{
				printf("CreateWaitableTimer failed (%d)\n", GetLastError());
				return 1;
			}

			printf ("Select sample rate using arrow keys; hit any other key when done... \n");
			printf ("\n     Ch1       Ch2       Ch3       Ch4\n");

			while (TRUE) {

				for (i = 0; i <=3 ; i++) {
					if (i == k) {
						printf(" ["); // Show selected channel to user using "[  ]"
					}
					else {
						printf("  ");
					}
					switch (iSrate[i]){ // Set and display the sample rate for all channels
						case 0: { printf("  1kS/s");Set_Sample_Rate (i,1); break;} 
						case 1: { printf("  2kS/s");Set_Sample_Rate (i,2); break;}
						case 2: { printf("  5kS/s");Set_Sample_Rate (i,5); break;}
						case 3: { printf(" 10kS/s");Set_Sample_Rate (i,10); break;}
						case 4: { printf(" 20kS/s");Set_Sample_Rate (i,20); break;}
						case 5: { printf(" 50kS/s");Set_Sample_Rate (i,50); break;}
						case 6: { printf("100kS/s");Set_Sample_Rate (i,100); break;}
						default:break;
					}
					if (i == k) {
						printf("]");
					}
					else {
						printf(" ");
					}
				}
				while( !_kbhit() ){
					// Set a timer to wait for 25 milli-seconds.
					// because a well behaved application tells windows when it is idling

					if (!SetWaitableTimer(hTimer, &liDueTime, 0, NULL, NULL, 0)) {
						printf("SetWaitableTimer failed (%d)\n", GetLastError());
						break;
					}
					// Wait for the timer.
					if (WaitForSingleObject(hTimer, INFINITE) != WAIT_OBJECT_0) {
						printf("WaitForSingleObject failed (%d)\n", GetLastError());
						break;
					}
				}
				printf ("\r");
				i = _getch();
				if ( i == 224) {
					i = _getch();
					if (i == 72 && iSrate[k] < 6) { // Up Arrow Key
						iSrate[k]++;
					}
					if (i == 80 && iSrate[k] > 0) { // Down Arrow Key
						iSrate[k]--;
					}
					if (i == 77 && k < 7) { // Right Arrow Key
						k++;
					}
					if (i == 75 && k > 0) { // Left Arrow Key
						k--;
					}
				}
				else{
					break;
				}
			}
			while (_kbhit()) {
				i = _getch(); // Throw away any keyboard hit
			}
			printf ("\n\n");
			printf ("DigIO : Cal Shunt[c/C]\n");
			printf ("\nUse '0' key to do Auto Zero\n");
			printf ("Use '1' key to toggle Cal Shunt\n");
			printf ("Use 'p' key to pause data acquisition\n");
			printf ("Use 'r' key to restart data acquisition\n");
			printf ("Use '+/-' keys to adjust calibration up and down\n");
			printf (" \nHit space bar to halt data acquisition.\n\n");
			printf ("\nDigIO   Ch1      Ch2      Ch3      Ch4\n");

			LV_Start_Acquisition(); // Starts a high priority data acquisition thread.

			while( TRUE ) {
				if (_kbhit()){
					i = _getch(); //  Handle the user key press
					switch (i) {
						case 0x30 : { // '0'
							Do_Autozero();
							break;
							}
						case 0x31 : { // '1'
							if (CAL_state){
								Do_Cal_Off();
								CAL_state = FALSE;
							}
							else {
								Do_Cal_On();
								CAL_state = TRUE;
							}
							break;
						}
						case '+' :
							Cal_Data++;
							Cal_Multiplier = float(Cal_Data) / 1000000000;
							break;
						case '-' :
							Cal_Data--;
							Cal_Multiplier = float(Cal_Data) / 1000000000;
							break;
						case 0x50 : // p or P
						case 0x70 : {
							Pause_Acquisition();
							break;
						}
						case 0x52 : // r or R
						case 0x72 : {
							if (Restart_Acquisition()) {
								li_Analog_Index[0] = 0;
								li_Analog_Index[1] = 0;
								li_Analog_Index[2] = 0;
								li_Analog_Index[3] = 0;
								Modulus_Multiples = 0;
							}
							break;
						}
						case 0x20 : {
							break;
						}
						default : break;
					}
					if (i == 0x20) {
						break; // Get out of the data acquisition loop here
					}
				}
				if (CAL_state) {
					printf ("C   ");
				}
				else {
					printf ("c   ");
				}	
				New_Data = FALSE;
				for (A_channel = 0 ;  A_channel <= 3 ; A_channel++ ){ // get each channel
					i = Get_DataPoint(A_channel,li_Analog_Index[A_channel]+10000);
					if ( i != FE813USB_ERROR_SAMPLE_NOT_READY) {
						if (i >= 0){ // If i < 0 there is an error , otherwise sample is valid
							New_Data = TRUE;
							iData_From_Channel[A_channel] = i;
							li_Analog_Index[A_channel] += 10000; // Get every 10000th sample
						}
						else {
							if ( i != FE813USB_ERROR_NO_TRANSFERS) { // We expect the NO_TRANSFERS error at start of acquisition
								printf ("\nError %d\n",i);
								i = -9999; // this code means an error
								// get here only if there is an error
							break;
							}
						}
					}
				}
				if (i == -9999) {
					break;
				}
				if (New_Data) {
					k = 0;
					for (i = 0 ; i < 4 ; i++) {
						printf ("%7.3fV ",((iData_From_Channel[i]-32768) * Cal_Multiplier)); // convert sample to volts
						if (li_Analog_Index[i] > Get_Sample_Modulus(i)) {
							k++;
						}
					}
					printf ("\r");
					if (k == 4) {// Don't let the sample index grow infinitely
						Modulus_Multiples++; // this can grow infinitely since it won't overflow for years
						Set_Sample_Boundary(Modulus_Multiples); // Tells the DLL to reduce sample index used to access data
						for (i = 0 ; i < 4 ; i++) {
							li_Analog_Index[i] -= Get_Sample_Modulus(i); // Reduce our sample index
						}
					}
				}
				else {	
//					printf(Get_String());
					printf ("\r");
						// wait because the data is not ready.
						// Set a timer to wait for 25 milli-seconds.
						// because a well behaved application tells windows when it is idling

					if (!SetWaitableTimer(hTimer, &liDueTime, 0, NULL, NULL, 0)) {
						printf("SetWaitableTimer failed (%d)\n", GetLastError());
						break;
					}
					// Wait for the timer.
					if (WaitForSingleObject(hTimer, INFINITE) != WAIT_OBJECT_0) {
						printf("WaitForSingleObject failed (%d)\n", GetLastError());
						break;
					}
				}
			}
			printf("\n");
			Stop_Acquisition();
			if (Set_Calibration_Data(Cal_Data) == TRUE) {
				printf ("\n\rCal Data set to : %d\n\r",Cal_Data);
			}
			else {
				printf ("\n\rWarning ... Cal Data not saved.\n\r"); 
			}
		}
		else
		{
			switch (i = Get_FE813USB_Status()) {
				case FE813USB_ERROR_NO_DEVICE : {
					printf ("No Device\n");
					break;
				}
				case FE813USB_ERROR_NO_DC_POWER : {
					printf ("Device is not powered\n");
					break;
				}
				default: {
					printf ("Unexpected Status %d\n",i);
				}
			}
		}

	printf ("\n\nHit <RETURN> to exit.\n");

	getchar(); // requires return key

	}

	return nRetCode;
}
